﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Core;

namespace HeuristicLab.EvolutionTracking {
  public interface IDirectedGraph : IItem { }
  public interface IDirectedGraph<TVertex> : IDirectedGraph
    where TVertex : class, IVertex {
    bool Contains(TVertex vertex);
    bool Any(Func<TVertex, bool> predicate); // graph contains any nodes matching the given predicate?
    void Clear(); // clear graph
    void AddVertex(TVertex vertex);
    void RemoveVertex(TVertex vertex); // remove node if contained in the graph
    List<TVertex> Nodes { get; }
  }
  public interface IDirectedGraph<TVertex, in TContent> : IDirectedGraph<TVertex>
    where TVertex : class, IVertex
    where TContent : class, IItem {
    bool Contains(TContent content); // graph contains node with given content?
  }
}
