﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.EvolutionTracking.Operators {
  [StorableClass]
  [Item("AfterSolutionCreatorOperator", "An operator that runs after the solution creator and performs additional actions.")]
  public class AfterSolutionCreatorOperator<TGraph, TVertex, TContent> : EvolutionTrackingOperator<TGraph, TVertex, TContent>
    where TGraph : class,IGenealogyGraph<TVertex, TContent>
    where TVertex : class,IGenealogyGraphNode<TContent>, new()
    where TContent : class,IItem {

    private string newIndividualParameterName;
    public string NewIndividualParameterName {
      get { return newIndividualParameterName; }
      set {
        if (newIndividualParameterName != null && Parameters.ContainsKey(newIndividualParameterName))
          Parameters.Remove(newIndividualParameterName);
        newIndividualParameterName = value;
        if (!Parameters.ContainsKey(newIndividualParameterName)) {
          Parameters.Add(new LookupParameter<TContent>(newIndividualParameterName));
        }
      }
    }

    public ILookupParameter<TContent> NewIndividualParameter {
      get { return (ILookupParameter<TContent>)Parameters[NewIndividualParameterName]; }
    }
    public TContent NewIndividual { get { return NewIndividualParameter.ActualValue; } }

    public override IOperation Apply() {
      var vertex = new TVertex {
        Content = NewIndividual
      };
      GenealogyGraph.AddVertex(vertex);
      return base.Apply();
    }
  }
}
