﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.EvolutionTracking {
  [StorableClass]
  [Item("AfterCrossoverOperator", "Performs an action after the crossover operator is applied.")]
  public class BeforeManipulatorOperator<TGraph, TVertex, TContent> : EvolutionTrackingOperator<TGraph, TVertex, TContent>
    where TGraph : class,IGenealogyGraph<TVertex, TContent>
    where TVertex : class,IGenealogyGraphNode<TContent>, new()
    where TContent : class,IItem {
    private string childParameterName;
    public string ChildParameterName {
      get { return childParameterName; }
      set {
        if (childParameterName != null && Parameters.ContainsKey(childParameterName)) Parameters.Remove(childParameterName);
        childParameterName = value;
        if (!Parameters.ContainsKey(childParameterName)) Parameters.Add(new LookupParameter<TContent>(childParameterName));
      }
    }

    #region parameter properties
    public ILookupParameter<TContent> ChildParameter {
      get { return (ILookupParameter<TContent>)Parameters[ChildParameterName]; }
    }
    #endregion
    protected BeforeManipulatorOperator(BeforeManipulatorOperator<TGraph, TVertex, TContent> original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new BeforeManipulatorOperator<TGraph, TVertex, TContent>(this, cloner);
    }

    public BeforeManipulatorOperator() { }

    public override IOperation Apply() {
      if (GenealogyGraph.Contains(ChildParameter.ActualValue)) {
        // if the individual has been affected by crossover before mutation, we keep it in the graph and feed a clone of it to the mutation operator
        ChildParameter.ActualValue = (TContent)ChildParameter.ActualValue.Clone();
      }
      return base.Apply();
    }
  }
}
