﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;

namespace HeuristicLab.EvolutionTracking {
  public class EvolutionTrackingOperator<TGraph, TVertex, TContent> : SingleSuccessorOperator
    where TGraph : class,IGenealogyGraph<TVertex, TContent>
    where TVertex : class,IGenealogyGraphNode<TContent>
    where TContent : class,IItem {
    // evolution tracking-related parameters
    private const string ResultsParameterName = "Results";
    private const string PopulationGraphParameterName = "PopulationGraph";
    private const string GenerationsParameterName = "Generations";

    public ILookupParameter<ResultCollection> ResultsParameter {
      get { return (ILookupParameter<ResultCollection>)Parameters[ResultsParameterName]; }
    }
    public ILookupParameter<IntValue> GenerationsParameter {
      get { return (ILookupParameter<IntValue>)Parameters[GenerationsParameterName]; }
    }
    public ResultCollection Results {
      get { return ResultsParameter.ActualValue; }
    }
    public IntValue Generations {
      get { return GenerationsParameter.ActualValue; }
    }
    public IGenealogyGraph<TVertex, TContent> GenealogyGraph {
      get {
        IResult result;
        if (!Results.ContainsKey(PopulationGraphParameterName)) {
          result = new Result(PopulationGraphParameterName, new GenealogyGraph<TVertex, TContent>());
          Results.Add(result);
        } else {
          result = Results[PopulationGraphParameterName];
        }
        var graph = (GenealogyGraph<TVertex, TContent>)result.Value;
        return graph;
      }
    }
    public EvolutionTrackingOperator() {
      Parameters.Add(new LookupParameter<IntValue>(GenerationsParameterName));
      Parameters.Add(new LookupParameter<ResultCollection>(ResultsParameterName));
    }
    protected EvolutionTrackingOperator(EvolutionTrackingOperator<TGraph, TVertex, TContent> original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new EvolutionTrackingOperator<TGraph, TVertex, TContent>(this, cloner);
    }
  }
}
